


/*
Christof Ermer
Regensburg
ChristofErmer@gmail.com
05.06.2017
*/

//------------------------------------------------------------------------------
// Programmed by Christof Ermer
//------------------------------------------------------------------------------

#define DATUM		"\n\r 11.07.2017 20:15"
#define SOFTWARE	"\n\r ** Braille Printer **"
#define AUTOR		"\n\r *** Christof Ermer ***"
#define VERSION		"\n\r V11.0"
#define HARDWARE	"\n\r ATM328"
#define IHAVENOBRAIN "\n\r I have no brain"	
#define USEYOURS	"\n\r Use your own"

#include <avr\sfr_defs.h>
#include <avr\io.h>
#include <avr\interrupt.h>
#include <avr\pgmspace.h>
#include <util\delay.h>
#include <avr\wdt.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>

#include "main.h"
#include "debug.h"
#include "uart328_16.h"
#include "Befehle.h"
#include "RingBuf.h"
#include "ParseRx.h"
#include "Braille.h"
#include "PrintInterface.h"
#include "Sprachdefinition.h"

#define UART_USE_ENABLED

#ifdef LED_MATRX_USED
	#include "max7219.h"
#endif

#ifdef I2C_USED
	#include "i2cmaster.h"
#endif

#ifdef  I2C_LCD_USED
	#include "i2clcd.h"
	#include "PCF8574.h"
#endif


/*
****** SOFTWARE SWITCHES  ********
*/
BRL_PARAMTER_TYPE gsParameter;
//gsParameter.u16PrinterSpeed = BRAILLE_SPEED_DEFAULT ;

// GLOBALE VARIABLEN 
volatile uint32_t gu32_Ticks;	 //  T2 TimerCounter
volatile uint8_t gu8MainStatus_Flags;

//volatile uint8_t gu8Modus;



// Interrupt Section
// ********************
ISR( TIMER2_OVF_vect )  //Dieser Vector steht in der iom328p.h
//gu32_Tick hochzaehlen und 244*/Sekunde ausfhren
// ********************
{// Interrupt Variablen mssen "volatile  & evtl static" deklariert sein
volatile static uint32_t u32MerkeT;
volatile static uint32_t u32BraileT;
volatile static uint32_t u32DisplayT;
if( ( (gu32_Ticks++) - u32MerkeT) > TICK_1000MS )
	{
	u32MerkeT = gu32_Ticks;	//neue Zeit merken
	gu8MainStatus_Flags |= STATUS_TICK_EVENT;
	};

if( ( (gu32_Ticks++) - u32DisplayT) > gsParameter.u16DisplaySpeed )
	{
	u32DisplayT = gu32_Ticks;	//neue Zeit merken
	gu8MainStatus_Flags |= STATUS_DISPLAY_TICK_EVENT;
	};

if( ( gu32_Ticks - u32BraileT) > gsParameter.u16PrinterSpeed )
	{
	u32BraileT = gu32_Ticks;	//neue Zeit merken
	gu8MainStatus_Flags |= STATUS_BRAILLE_TICK_EVENT;
	};
//UART ist jetz in Main ausgelagert	
};



// *************
int main(void)
// *************
{
uint8_t u8NN=0;
uint8_t u8WW=0;
//int8_t iRet=0;
uint32_t u32MainloopCnt=0;

INTERN_LED_DDRB |= LED1_BLINK_BIT;
INIT_BRAILLE_LED();


RingBuf_Init( &gsCodeSetStrBuf, gu8aCodeSetStr, PARSE_RINGBUFF_SIZE); // Ringgbuf.h RINGBUFF_SIZE 256
RingBuf_Init( &gsBrailleStrBuf, gu8aBrailleStr, BRAILLE_RINGBUF_SIZE);
RingBuf_Bytes_Available(&gsCodeSetStrBuf); // Beim ersten mal war der Wert falsch
RingBuf_Bytes_Available( &gsBrailleStrBuf); // Beim ersten mal war der Wert falsch

#ifdef LED_MATRX_USED
MAX7219_Init();
MAX7219_Clear();
#endif

#ifdef I2C_USED
// TWI initialisieren
TWI_DDRX |= TWI_BITS; //I2C DDRC=x030
i2c_init();
#endif

#ifdef I2C_LCD_USED
	// LCD initialisieren
lcd_init();
// LCD-Anzeige loeschen
lcd_command(LCD_CLEAR);
// Hintergrundbeleuchtung an
lcd_backlight(ON);
// Text auf LCD ausgeben
// MIT RAM , als falsch
strcpy_P( gcaStr, PSTR("* I2C LCD *") );
	lcd_printlc(1,1,(unsigned char *)gcaStr);
strcpy_P( gcaStr, PSTR("*Christof Ermer*"));
	lcd_printlc(2,1,(unsigned char *)gcaStr);
#endif

#ifdef UART_USE_ENABLED
uart_init( UART_BAUD_SELECT_DOUBLE_SPEED( UART_BAUD_RATE, F_CPU) );
uart_flush();
CMD_StringReset();
#endif

wdt_reset();   //WATCHDOG!
wdt_enable(5); //500mS Totmannknopfsei();

//--------------------------
// INIT 
gsParameter.u16PrinterSpeed = BRAILLE_SPEED_DEFAULT;
gsParameter.u16DisplaySpeed = DISPLAY_UPDATE_SPEED_DEFAULT;

//--------------------------
//TickCounter Starten
TCCR2B = (1 << CS22) | (1 << CS21);	//Vorteiler //256 einschalten
TIMSK2 = (1 << TOIE2);	//Interrrupt aktivieren

sei();	//Global Interrupt Freischalten
wdt_reset();

#ifdef UART_USE_ENABLED
uart_puts_p( PSTR("\n\r") );
uart_puts_p( PSTR( SOFTWARE ) );
uart_puts_p( PSTR( HARDWARE ));
wdt_reset();
uart_puts_p( PSTR( AUTOR ) );
uart_puts_p( PSTR( DATUM )  );
uart_puts_p( PSTR( VERSION ) );
#endif
//uart_puts_p( PSTR( IHAVENOBRAIN ) );
//uart_puts_p( PSTR( USEYOURS ) );
uart_puts_p( PSTR("\n\r") );
wdt_reset();


ResetCodeSet(); //gu8Analyse_Flags=0;
ResetBraille_Buf(); //gu8BrailleStatus_Flags=0;
gu8PrinterStatus_Flags=0;

for(u8WW=0; u8WW < (3*6); u8WW++)
	{
	wdt_reset();	//WATCHDOG!
	CLR_BRAILLE_LED();
	SET_BRAILLE_LED( 1<< u8NN);
	u8NN++;
	u8NN %= LED_BRAILLE_LEDCNT;
	_delay_ms(ANIMATION_DELAY_MS);
	wdt_reset();	//WATCHDOG!
	};



for(u8WW=0; u8WW < 3; u8WW++)
	{
	wdt_reset();	//WATCHDOG!
	CLR_BRAILLE_LED();
	SET_BRAILLE_LED( BRAILLE_GERADE_OBEN );  // Waagerechter Strich
	_delay_ms(ANIMATION_DELAY_MS);
	
	CLR_BRAILLE_LED();
	SET_BRAILLE_LED( BRAILLE_GERADE_MITTE );  // Waagerechter Strich
	_delay_ms(ANIMATION_DELAY_MS);
	
	CLR_BRAILLE_LED();
	SET_BRAILLE_LED(  BRAILLE_GERADE_UNTEN );  // Waagerechter Strich
	_delay_ms(ANIMATION_DELAY_MS);
	wdt_reset();	//WATCHDOG!
	};
	
CLR_BRAILLE_LED();
SET_BRAILLE_LED( BRAILLE_GERADE_MITTE );  // Wagerechter Stricch

for(;;)  //ENSLESS
	{
//Toggle_Bit(1);
	wdt_reset();	//WATCHDOG!
	u32MainloopCnt++;
// ============================================================	
// NEU; JETZT IN MAIN, wenn Rx da ist. Keine Tick Blockierung mehr
	UART_RX_Check();  //RX UART -->Parse.c
	ScanRxStrToBraille();
// ============================================================	
	if( gu8MainStatus_Flags & STATUS_TICK_EVENT )
		{// ab jetzt langsame ~1Hz Action
		gu8MainStatus_Flags &= ~STATUS_TICK_EVENT;
		INTERN_LED_PORTB ^= LED1_BLINK_BIT;	

/*

*/		
#ifdef UART_USE_ENABLED
/*		
		uart_puts_p( PSTR("\n\r Loop:") );
		uart_puts( ultoa( u32MainloopCnt, gcaStr,  10) );	
		uart_puts_p( PSTR("\n\r") );
*/

/*

		uart_puts_p( PSTR("\n\r CodeSetBuf:") );
		uart_puts( CopyRingbufToStr( &gsCodeSetStrBuf, gcaStr, 16 ) );						 

		uart_puts_p( PSTR("\n\r CodeSetDec:") );
		uart_puts( (char*)BYTES20_ToDecStr( (unsigned char*)gcaStr) ); 		
	 
		uart_puts_p( PSTR("\n\r BrailleStr:") );
		uart_puts( CopyRingbufToStr( &gsBrailleStrBuf, gcaStr, 16 ) );	
					 
		uart_puts_p( PSTR("\n\r AFlag:") );
		uart_puts(  ByteToBin( gu8Analyse_Flags )  );
		 
		uart_puts_p( PSTR("\n\r") );
*/
		//uart_putc(13);
#endif
		
		u32MainloopCnt=0;
		};		

// ============================================================	
	if(gu8MainStatus_Flags & STATUS_BRAILLE_TICK_EVENT)	
		{		
		gu8MainStatus_Flags &= ~STATUS_BRAILLE_TICK_EVENT;		

		if( !(gu8PrinterStatus_Flags & PRINTER_BUSY )  )
			{		
			if( !Check_PrintLines_Available() )
				{
	Toggle_Bit( 10 );		
				Aktion_PaperChanged();
				
				}
			else
				{
				Trigger_BRL_Printer();
				};//Check_PrintLines_Available
			};//STATUS_PRINTER_BUSY
		};//STATUS_BRAILLE_TICK_EVENT

// ============================================================	
	if(gu8MainStatus_Flags & STATUS_DISPLAY_TICK_EVENT)	
		{		
		gu8MainStatus_Flags &= ~STATUS_DISPLAY_TICK_EVENT;		
		#ifdef LED_MATRX_USED
		MAX7219_Write(1, gu8BefehlStatus ); //gu8MainStatus_Flags
		MAX7219_Write(2, gu8RxParse_State_Flags);
		MAX7219_Write(3, gu8Analyse_Flags);
		MAX7219_Write(4, gu8BrailleStatus_Flags);
		MAX7219_Write(5, gu8PrinterStatus_Flags);
		//MAX7219_Write(6, gu8EncodedBrailleChar);

		MAX7219_Write(7, (gu8ActualPrinterBrailleChar & 0x07) );
		MAX7219_Write(8, ((gu8ActualPrinterBrailleChar >> 3) & 0x07) );
		//MAX7219_Write(8, ((1<< u8NN % 8)) );
		//u8NN++;
		#endif
		#ifdef I2C_LCD_USED	
		//CopyRingbufToStr(&gsCodeSetStrBuf, gcaStr, 16 );
		//LCD_L1_StrDebug( gcaStr );
		//ultoa( u32MainloopCnt, gcaNumStr,  10);
		//LCD_L2_StrDebug( gcaNumStr  );	

		lcd_command(LCD_CLEAR);// LCD-Anzeige loeschen
		lcd_printlc(1,1,(unsigned char *) gcaLCD_L1);  //LCD_L1_StrDebug()
		lcd_printlc(2,1,(unsigned char *) gcaLCD_L2);	//LCD_L2_StrDebug()
		#endif	
		};
	};
}; // main

//ultoa( u32MainloopCnt, gcaStr, 10 );
//lcd_printlr(2,8,(unsigned char *)gcaStr);	
//strcpy_P( gcaStr, PSTR("* I2C LCD *") );
